local Een = CreateFrame("FRAME"); -- Need a frame to respond to events
Een:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
Een:RegisterEvent("UPDATE_CHAT_COLOR"); -- Fired when the colour for a chat channel is changed
Een:RegisterEvent("LOADING_SCREEN_DISABLED"); -- ¯\_(ツ)_/¯ Prolly fires when the Loading screen is hidden/closed/removed/undrawn/etc.
Een:RegisterEvent("UPDATE_CHAT_COLOR_NAME_BY_CLASS"); -- Fires when settings for the per-class colour of character names in chat are set
Een_SaveMessage = false;

local chatColours = { };

-- found this part on http://wowwiki.wikia.com/wiki/Creating_simple_pop-up_dialog_boxes
-- START
StaticPopupDialogs["EEN_CHAT_RELOAD"] = {
  text = "This will require the UI to be reloaded.",
  button1 = "Okay",
  button2 = "No, Later",
  OnAccept = function()
      ReloadUI()
  end,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  preferredIndex = 3,  -- avoid some UI taint, see http://www.wowace.com/announcements/how-to-avoid-some-ui-taint/
}
-- END of WoWWiki's code

function Een_Save_Chat()
	for i=1,10 do -- there can only be 10 chat tabs so far, so count 1 thru 10
		local f = _G["ChatFrame"..i]; -- get the object that is the global named ChatFrame1, ChatFrame2, ChatFrame3, etc.
		local point, relativeTo, relativePoint, xOfs, yOfs = f:GetPoint(); -- gets the location info for the ChatFrame
		local name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable = GetChatWindowInfo(i); -- get the other info about the ChatFrame
		local DefaultMessages = { GetChatWindowMessages(f:GetID())}; -- get what messages(i.e. system, player, whisper, tradeskills, etc.)
		local DefaultChannels = { GetChatWindowChannels(f:GetID())}; -- get the channels(i.e. general, localdefense, trade, etc.)

-- Get frame parent in full, stored in 'tmpParentName'
		tmpParent = f:GetParent():GetName();
		
-- Store all that info in the array/table/thing
		Een_ChatFrame[i] = {f:GetWidth(), f:GetHeight(), point, point, xOfs, yOfs, name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable, DefaultMessages, DefaultChannels, tmpParent};
	end
	
-- save chat channel colours from the temp variable to the WTF
	Een_ChannelColours = chatColours;
	
--Check if messages can be given and do so appropriately	
	if not Een_Quiet then
		if not Een_Msg then 
			message("Chat Tab data saved.");
		else
			print("*** Chat Tab data saved.");
		end
	end
end

function Een_Assimilate_Chat(args)

	for i=1,10 do
		local f = _G["ChatFrame"..i];
		local tmp_ChatFrame = args[i];
		if tmp_ChatFrame then
			tmpParent = _G[tmp_ChatFrame[19]]; -- get the object that has the name of the ChatFrame's parent
			f:SetParent(tmpParent); -- set the ChatFrame's parent
	-- Remove old chat message cfg and replace with new one		
			local tmp_OldChatMsgType = { GetChatWindowMessages(f:GetID())};
			for k=1,#tmp_OldChatMsgType do
				RemoveChatWindowMessages(i, tmp_OldChatMsgType[k]);
			end
			local tmp_ChatMsgType = tmp_ChatFrame[17];
			for j=1,#tmp_ChatFrame[17] do
				AddChatWindowMessages(i, tmp_ChatMsgType[j]);
			end
			
	-- Remove old chat channel cfg and replace with new one	
			if tmp_ChatFrame[18] then
				local tmp_OldChatChannelType = { GetChatWindowChannels(f:GetID())};
				for k=1,#tmp_OldChatChannelType do
					RemoveChatWindowChannel(i, tmp_OldChatChannelType[k]);
				end
				local tmp_ChatChannelType = tmp_ChatFrame[18];
				for j=1,#tmp_ChatFrame[18] do
					AddChatWindowChannel(i, tmp_ChatChannelType[j]);
				end
			end

	-- Set the other infos
			SetChatWindowName(i,tmp_ChatFrame[7]);
			SetChatWindowSize(i,tmp_ChatFrame[8]);
			SetChatWindowColor(i,tmp_ChatFrame[9],tmp_ChatFrame[10],tmp_ChatFrame[11]);
			SetChatWindowAlpha(i,tmp_ChatFrame[12]);
			SetChatWindowShown(i,tmp_ChatFrame[13]);
			SetChatWindowLocked(i,tmp_ChatFrame[14]);
			SetChatWindowDocked(i,tmp_ChatFrame[15]);
			SetChatWindowUninteractable(i,tmp_ChatFrame[16]);
			if f:IsMovable() then -- I don't remember when this wouldn't be true for the ChatFrames, this might need to be removed?
				f:ClearAllPoints();
				f:SetWidth(tmp_ChatFrame[1]);
				f:SetHeight(tmp_ChatFrame[2]);

	-- We only need to pass the one reference point and the x & y from that point, it defaults to using the parent and matches the
	-- reference point(i.e. topleft on the parent to topleft of the Chat Frame. I left the old command anyway
	--			f:SetPoint(tmp_ChatFrame[3], tmpParent, tmp_ChatFrame[4], tmp_ChatFrame[5], tmp_ChatFrame[6]);
				f:SetPoint(tmp_ChatFrame[3], tmp_ChatFrame[5], tmp_ChatFrame[6]);
				f:SetUserPlaced(true);
			end
		end
	end
	
	-- Set the Colours for the ChatFrame's messages from the WTF
--	for i,line in pairs(Een_ChannelColours) do
--		ChangeChatColor(line[1], line[2], line[3], line[4]); -- channel colours
--		SetChatColorNameByClass(line[1], line[5]); -- set class colour shown/not shown
--	end
	
--Check if messages can be given and do so appropriately	
	if not Een_Msg and not Een_Quiet then 
		message("Chat settings loaded. Please type '/reload' to reflect the change.");
	elseif not Een_Quiet then
		print("*** Chat settings loaded. Please type '/reload' to reflect the change.");
	end;
end

function Een_Reset()
	for i=1,10 do
		local f = _G["ChatFrame"..i];
		local point, relativeTo, relativePoint, xOfs, yOfs = f:GetPoint();
		local name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable = GetChatWindowInfo(i);
		local DefaultMessages = { GetChatWindowMessages(f:GetID())};
		local DefaultChannels = { GetChatWindowChannels(f:GetID())};
		-- checking so that only the chat tabs with channels and messages are reset and docked
		-- If this isn't here, any ChatFrames that aren't used will be made visible and docked, but they won't have settings for messages/channels and will be empty
		if table.getn(DefaultMessages) ~= 0 or table.getn(DefaultChannels) ~= 0 then
			Een_ChatFrame[i] = {f:GetWidth(), f:GetHeight(), "CENTER", "CENTER", 0, 0, name, fontSize, r, g, b, alpha, shown, true, i, false, DefaultMessages, DefaultChannels, _G["ChatFrame1"]:GetParent():GetName()};
		else -- resetting the empty chat tabs(i.e. no channels or messages assigned)
			Een_ChatFrame[i] = {f:GetWidth(), f:GetHeight(), "CENTER", "CENTER", 0, 0, name, fontSize, r, g, b, alpha, shown, true, nil, true, DefaultMessages, DefaultChannels, "UIParent"};
		end
	end
	
-- Call Chat Load to move the panels to their reset positions
-- also this stops the Chat Load from printing anything as we have a specific reset message we want to print
	tmpQuiet = Een_Quiet;
	Een_Quiet = true;
	Een_Assimilate_Chat(Een_ChatFrame);
	Een_Quiet = tmpQuiet;

	StaticPopup_Show ("EEN_CHAT_RELOAD");
	
--Check if messages can be given and do so appropriately	
	if not Een_Msg and not Een_Quiet then 
		message("Een's Chat Window backup has reset the\nposition of all chat tabs, and docked them all too.");
	elseif not Een_Quiet then
		print("*** Een's Chat Window backup has reset the position of all chat tabs, and docked them all too.");
	end;
end

-- used to count entries in a table, was having issues with other methods and this seemed to work
function Een_TableCount(table) 
	local tmpCounter = 0
	for i,line in pairs(table) do
		tmpCounter = tmpCounter + 1;
	end
	return tmpCounter;
end

function Een_Debug()
-- For the Debugging, should be empty unless I forgot something
end

function Een:OnEvent(event, arg1, arg2, arg3, arg4)
	if event == "ADDON_LOADED" then 
		if arg1 == "Een" then -- Fire when this addon loads
			if Een_ChatFrame == nil then -- is the main table nil(happens when addon is loaded without WTF data)
				Een_ChatFrame = { };
				printInstructions();
				print("|cFFFF0000 EEN CHAT WINDOW BACKUP |r|cFFFFFF00HASN'T BEEN SAVED YET. PLEASE USE '/een save' TO SAVE YOUR CHAT TABS|r");
				Een_SaveMessage = true;

		-- these were moved here so that they didn't always overwrite the settings everytime the addon loaded :/
				Een_AutoLoad = true; -- true = autoload, false = don't autoload
				Een_Msg = true; -- true = no popups, false = popup messages
				Een_Quiet = false; -- true = no messages, false = print/popup messages
			elseif Een_ChatFrame[1] ~= nil then -- this only prints that it has loaded, the addon is really fired by the Chat Load below
				if Een_AutoLoad then 
					print("Een's addon loaded.");
				end
			else -- in case anything else happens(extremely rare) just print the instructions
				printInstructions();
			end
			if Een_ChannelColours == nil then -- if the colours table is nil(happens when addon is loaded without WTF data)
				Een_ChannelColours = { } ;
			end
		end 
	elseif event == "LOADING_SCREEN_DISABLED" then
		if Een_TableCount(Een_ChatFrame) == 0 then -- This will warn the player to do the '/een save' command everytime the load screen finishes
			Een_ChatFrame = { };
			if not Een_SaveMessage then
				print("|cFFFF0000 EEN CHAT WINDOW BACKUP |r|cFFFFFF00HASN'T BEEN SAVED YET. PLEASE USE '/een save' TO SAVE YOUR CHAT TABS|r");
			end
			Een_SaveMessage = not Een_SaveMessage;
		else
			if Een_AutoLoad then
				Een_Assimilate_Chat(Een_ChatFrame); -- call the function to load the settings
			end
		end
		if Een_TableCount(Een_ChannelColours) == 0 then 
			print(table.getn(Een_ChannelColours));
			print("|cFFFF0000T|cFFFF7F00H|cFFFFFF00E|cFF00FF00R|cFF0000FFE |cFF4B0082I|cFF8F00FFS |cFFFF0000N|cFFFF7F00O |cFFFFFF00C|cFF00FF00H|cFF0000FFA|cFF4B0082T |cFF8F00FFC|cFFFF0000O|cFFFF7F00L|cFFFFFF00O|cFF00FF00U|cFF0000FFR |cFF4B0082D|cFF8F00FFA|cFFFF0000T|cFFFF7F00A |cFFFFFF00S|cFF00FF00A|cFF0000FFV|cFF4B0082E|cFF8F00FFD|cFFFF0000. |cFFFF7F00P|cFFFFFF00L|cFF00FF00E|cFF0000FFA|cFF4B0082S|cFF8F00FFE |cFFFF0000C|cFFFF7F00H|cFFFFFF00A|cFF00FF00N|cFF0000FFG|cFF4B0082E |cFF8F00FFO|cFFFF0000N|cFFFF7F00E |cFFFFFF00O|cFF00FF00F |cFF0000FFT|cFF4B0082H|cFF8F00FFE |cFFFF0000C|cFFFF7F00O|cFFFFFF00L|cFF00FF00O|cFF0000FFU|cFF4B0082R|cFF8F00FFS |cFFFF0000A|cFFFF7F00N|cFFFFFF00D |cFF00FF00T|cFF0000FFH|cFF4B0082E|cFF8F00FFN |cFFFF0000R|cFFFF7F00U|cFFFFFF00N|cFFFFFFFF'/een save'");
		end
	elseif event == "UPDATE_CHAT_COLOR"  then -- this fires every time one of the channels/messages gets its colour changed 
		local channelColours = { };           -- and will save the colours in a temp variable so that it doesn't overwrite the saved colours
		channelColours[1] = arg1;             -- unfortunately this needs to be done in a temp variable since the colours can't be read
		channelColours[2] = arg2;             -- outside of the event which fires when they are updated
		channelColours[3] = arg3;
		channelColours[4] = arg4;
		chatColours[arg1] = channelColours;
 	elseif event == "UPDATE_CHAT_COLOR_NAME_BY_CLASS" then -- Fires when settings for the per-class colour of character names in chat are set	
		local channelColours = chatColours[arg1];          -- same temp variable reasoning as above
		if channelColours ~= nil then
			channelColours[5] = arg2;
			chatColours[arg1] = channelColours;
		end
	end
end

function printInstructions()
	print("Welcome to Een's Chat Window backup");
	print("Usage:");
	print("/een info - Show this info and the current settings");
	print("/een save - Save the current character's chat windows");
	print("/een load - Load the backup");
	print("/een autoload - toggles the auto-magic loading of the chat window locations(default is on)");
	print("/een msg - togles the popup telling you to reloadui(default hides the message)");
	print("/een reset - should reset the chat tabs to be docked, have a parent of UIParent, be interactible and be locked");
	print("/een ver - toggles verbose mode(showing messages at all)(default shows messages)");
end

Een:SetScript("OnEvent", Een.OnEvent);

SLASH_EEN1 = "/een";
SlashCmdList["EEN"] = function(msg)
	if msg == 'save' then
		Een_Save_Chat();
	elseif msg == 'load' and Een_ChatFrame[1] ~= nil then
		Een_Assimilate_Chat(Een_ChatFrame);
	elseif msg == 'autoload' then
		Een_AutoLoad = not Een_AutoLoad;
	--print autoload ifo
		local tmpBoolStr = "now";
		if not Een_AutoLoad then tmpBoolStr = "not" end;
		print("*** Een's Chat Window backup", tmpBoolStr, "set to autoload.");
	elseif msg == 'msg' then
		Een_Msg = not Een_Msg;
	--print pop-up status info
		local tmpBoolStr = "print";
		if not Een_Msg then tmpBoolStr = "pop-up" end;
		print("*** Een's Chat Window backup is now set to", tmpBoolStr, "the message.");
	elseif msg == 'reset' then
		Een_Reset();
	elseif msg == 'debug' then
		Een_Debug();
	elseif msg == 'ver' then
		print("*** Een's Chat Window backup has verbose mode set to ", Een_Quiet);
		Een_Quiet = not Een_Quiet;
	elseif msg == 'info' then
	--print instructions
		printInstructions();
	--print autoload ifo
		local tmpBoolStr = "now";
		if not Een_AutoLoad then tmpBoolStr = "not" end;
		print("*** Een's Chat Window backup", tmpBoolStr, "set to autoload.");
	--print pop-up status info
		local tmpBoolStr = "print";
		if not Een_Msg then tmpBoolStr = "pop-up" end;
		print("*** Een's Chat Window backup is now set to", tmpBoolStr, "the message.");
	--print verbose mode info
		local tmpBoolStr = not Een_Quiet;
		print("*** Een's Chat Window backup has verbose mode set to", tmpBoolStr);
	else
		printInstructions();
	end
end